/* texttospeech
 *
 * texttospeech.c - texttospeech plugin for purple using mbrola and txt2pho
 *
 * Copyright  2006 Tobias Gehrig <tobias@gehrignet.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 */

#include "debug.h"
#include <purple.h>
#include "signals.h"
#include "util.h"
#include "plugin.h"
#include "version.h"
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <string.h>
#include <unistd.h>

#ifdef HAVE_CONFIG_H
#include <purple-texttospeech-config.h>
#endif

static gboolean
say_text(PurpleAccount *account, char *sender, char *message,
	 int flags, void *data)
{
  if (message == NULL)
    return FALSE;
  {
    int pid;
    int pc[2]; /* Parent to child pipe */

    /* Make pipes */
    if( pipe(pc) < 0) {
      purple_debug(PURPLE_DEBUG_ERROR, "tts", "Can't make pipe\n");
      return FALSE;
    }
    
    /* Create a child to run command. */
    switch( pid = fork() ) {
    case -1:
      purple_debug(PURPLE_DEBUG_ERROR, "tts", "Can't fork\n");
      break;
    case 0:
      {
	/* Child. */
	char *args[4];
	close(0); /* Close current stdin. */
	dup( pc[0]); /* Make stdin come from read
			end of pipe. */
	close( pc[1]);

	//snprintf(buffer, sizeof(buffer), "festival -b '(SayText \"%s\")'", msg);
	//snprintf(buffer, sizeof(buffer), "/home/tobias/bin/sag_was \"%s\"", msg);
	args[0] = "sh";
	args[1] = "-c";
	args[2] = "/home/tobias/bin/sag_was";
	args[3] = NULL;
	
	execvp(args[0], args);
	
	_exit(0);
      }
    default:
      {
	/* Parent. */
	char* msg = purple_markup_strip_html(message);
	purple_debug(PURPLE_DEBUG_INFO, "tts", "message: %s\n", msg);

	/* Close what we don't need. */
	write(pc[1], msg, strlen(msg));
	close(pc[1]);
	g_free(msg);
      }
    }
  }
  return FALSE;
}

static gboolean
plugin_load(PurplePlugin *plugin)
{
	void *conv_handle = purple_conversations_get_handle();
	purple_signal_connect(conv_handle, "received-im-msg",
			    plugin, PURPLE_CALLBACK(say_text), NULL);
	purple_debug(PURPLE_DEBUG_INFO, "tts", "tts plugin loaded.\n");

	return TRUE;
}

static gboolean
plugin_unload(PurplePlugin *plugin)
{
	purple_debug(PURPLE_DEBUG_INFO, "tts", "tts plugin unloaded.\n");

	return TRUE;
}

static PurplePluginInfo info =
{
	PURPLE_PLUGIN_MAGIC,
	PURPLE_MAJOR_VERSION,
	PURPLE_MINOR_VERSION,
	PURPLE_PLUGIN_STANDARD,                             /**< type           */
	NULL,                                             /**< ui_requirement */
	0,                                                /**< flags          */
	NULL,                                             /**< dependencies   */
	PURPLE_PRIORITY_DEFAULT,                            /**< priority       */

	"core-tgehrig-texttospeech",                                             /**< id             */
	"Text-to-speech plugin",                          /**< name           */
        TTS_VERSION,                                          /**< version        */
	                                                  /**  summary        */
	"synthesizes the IM message through mbrola.",
	                                                  /**  description    */
	"This Purple plugin feeds received text to a text reader along with the desired language.",
	"Tobias Gehrig <tobias@gehrignet.de>",        /**< author         */
	"http://www.gehrignet.de",                                          /**< homepage       */

	plugin_load,                                      /**< load           */
	plugin_unload,                                    /**< unload         */
	NULL,                                             /**< destroy        */

	NULL,                                             /**< ui_info        */
	NULL,                                             /**< extra_info     */
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL
};

static void
init_plugin(PurplePlugin *plugin)
{
}

PURPLE_INIT_PLUGIN(tts, init_plugin, info)
